
/*
 * original kernel code by Andre Hedrick
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/io.h>

#include "atapci.h"

static struct aec_dev {
	u16 device;
	char *name;
} aec_devs[] = {
	{ PCI_DEVICE_ID_ARTOP_ATP850UF, "6210" },
	{ PCI_DEVICE_ID_ARTOP_ATP860,   "6260" },
	{ PCI_DEVICE_ID_ARTOP_ATP860R,  "6260 (with ROM)" },
	{ PCI_DEVICE_ID_ARTOP_ATP865,   "6280" },
	{ PCI_DEVICE_ID_ARTOP_ATP865R,  "6280 (with ROM)" },
};

int aec_info(char *buffer, struct pci_dev *dev)
{
	u32 bibma = pci_resource_start(dev, 4);
	u8 c0 = 0, c1 = 0;
	u8 art = 0, uart = 0;
	unsigned int i;
	char *p = buffer;

	for (i = 0; i < ARRAY_SIZE(aec_devs); i++)
		if (dev->device_id == aec_devs[i].device)
			break;

	p += sprintf(p, "\n\t\t\t\tAEC%s Chipset.", aec_devs[i].name);

        /*
         * at that point bibma+0x2 et bibma+0xa are byte registers
         * to investigate:
         */
	if(iopl(3))
		need_root();
	c0 = inb_p((unsigned short)bibma + 0x02);
	c1 = inb_p((unsigned short)bibma + 0x0a);
	iopl(0);

	p += sprintf(p, "--------------- Primary Channel ---------------- Secondary Channel -------------\n");
	pci_read_config_byte(dev, 0x4a, &art);
	p += sprintf(p, "                %sabled                         %sabled\n",
		(art&0x02)?" en":"dis",
		(art&0x04)?" en":"dis");
	p += sprintf(p, "--------------- drive0 --------- drive1 -------- drive0 ---------- drive1 ------\n");
	p += sprintf(p, "DMA enabled:    %s              %s             %s               %s\n",
		(c0&0x20)?"yes":"no ",
		(c0&0x40)?"yes":"no ",
		(c1&0x20)?"yes":"no ",
		(c1&0x40)?"yes":"no ");

	switch(dev->device_id) {
		case PCI_DEVICE_ID_ARTOP_ATP850UF:
			pci_read_config_byte(dev, 0x54, &art);
			p += sprintf(p, "DMA Mode:       %s(%s)          %s(%s)         %s(%s)           %s(%s)\n",
				(c0&0x20)?((art&0x03)?"UDMA":" DMA"):" PIO",
				(art&0x02)?"2":(art&0x01)?"1":"0",
				(c0&0x40)?((art&0x0c)?"UDMA":" DMA"):" PIO",
				(art&0x08)?"2":(art&0x04)?"1":"0",
				(c1&0x20)?((art&0x30)?"UDMA":" DMA"):" PIO",
				(art&0x20)?"2":(art&0x10)?"1":"0",
				(c1&0x40)?((art&0xc0)?"UDMA":" DMA"):" PIO",
				(art&0x80)?"2":(art&0x40)?"1":"0");
			pci_read_config_byte(dev, 0x40, &art);
			p += sprintf(p, "Active:         0x%02x", art);
			pci_read_config_byte(dev, 0x42, &art);
			p += sprintf(p, "             0x%02x", art);
			pci_read_config_byte(dev, 0x44, &art);
			p += sprintf(p, "            0x%02x", art);
			pci_read_config_byte(dev, 0x46, &art);
			p += sprintf(p, "              0x%02x\n", art);
			pci_read_config_byte(dev, 0x41, &art);
			p += sprintf(p, "Recovery:       0x%02x", art);
			pci_read_config_byte(dev, 0x43, &art);
			p += sprintf(p, "             0x%02x", art);
			pci_read_config_byte(dev, 0x45, &art);
			p += sprintf(p, "            0x%02x", art);
			pci_read_config_byte(dev, 0x47, &art);
			p += sprintf(p, "              0x%02x\n", art);
			break;
		case PCI_DEVICE_ID_ARTOP_ATP860:
		case PCI_DEVICE_ID_ARTOP_ATP860R:
			pci_read_config_byte(dev, 0x44, &art);
			p += sprintf(p, "DMA Mode:       %s(%s)          %s(%s)",
				(c0&0x20)?((art&0x07)?"UDMA":" DMA"):" PIO",
				((art&0x06)==0x06)?"4":
				((art&0x05)==0x05)?"4":
				((art&0x04)==0x04)?"3":
				((art&0x03)==0x03)?"2":
				((art&0x02)==0x02)?"1":
				((art&0x01)==0x01)?"0":"?",
				(c0&0x40)?((art&0x70)?"UDMA":" DMA"):" PIO",
				((art&0x60)==0x60)?"4":
				((art&0x50)==0x50)?"4":
				((art&0x40)==0x40)?"3":
				((art&0x30)==0x30)?"2":
				((art&0x20)==0x20)?"1":
				((art&0x10)==0x10)?"0":"?");
			pci_read_config_byte(dev, 0x45, &art);
			p += sprintf(p, "         %s(%s)           %s(%s)\n",
				(c1&0x20)?((art&0x07)?"UDMA":" DMA"):" PIO",
				((art&0x06)==0x06)?"4":
				((art&0x05)==0x05)?"4":
				((art&0x04)==0x04)?"3":
				((art&0x03)==0x03)?"2":
				((art&0x02)==0x02)?"1":
				((art&0x01)==0x01)?"0":"?",
				(c1&0x40)?((art&0x70)?"UDMA":" DMA"):" PIO",
				((art&0x60)==0x60)?"4":
				((art&0x50)==0x50)?"4":
				((art&0x40)==0x40)?"3":
				((art&0x30)==0x30)?"2":
				((art&0x20)==0x20)?"1":
				((art&0x10)==0x10)?"0":"?");
			pci_read_config_byte(dev, 0x40, &art);
			p += sprintf(p, "Active:         0x%02x", art >> 4);
			pci_read_config_byte(dev, 0x41, &art);
			p += sprintf(p, "             0x%02x", art >> 4);
			pci_read_config_byte(dev, 0x42, &art);
			p += sprintf(p, "            0x%02x", art >> 4);
			pci_read_config_byte(dev, 0x43, &art);
			p += sprintf(p, "              0x%02x\n", art >> 4);
			pci_read_config_byte(dev, 0x40, &art);
			p += sprintf(p, "Recovery:       0x%02x", art & 0xf);
			pci_read_config_byte(dev, 0x41, &art);
			p += sprintf(p, "             0x%02x", art & 0xf);
			pci_read_config_byte(dev, 0x42, &art);
			p += sprintf(p, "            0x%02x", art & 0xf);
			pci_read_config_byte(dev, 0x43, &art);
			p += sprintf(p, "              0x%02x\n", art & 0xf);
			pci_read_config_byte(dev, 0x49, &uart);
			p += sprintf(p, "reg49h = 0x%02x ", uart);
			pci_read_config_byte(dev, 0x4a, &uart);
			p += sprintf(p, "reg4ah = 0x%02x\n", uart);
			break;
		case PCI_DEVICE_ID_ARTOP_ATP865:
		case PCI_DEVICE_ID_ARTOP_ATP865R:
			pci_read_config_byte(dev, 0x44, &art);
			p += sprintf(p, "DMA Mode:       %s(%u)          %s(%u)",
				(c0&0x20)?((art&0x0f)?"UDMA":" DMA"):" PIO",
				(art & 0x0f) - 1,
				(c0&0x40)?((art&0xf0)?"UDMA":" DMA"):" PIO",
				(art >> 4) - 1);
			pci_read_config_byte(dev, 0x45, &art);
			p += sprintf(p, "         %s(%u)          %s(%u)\n",
				(c0&0x20)?((art&0x0f)?"UDMA":" DMA"):" PIO",
				(art & 0x0f) - 1,
				(c0&0x40)?((art&0xf0)?"UDMA":" DMA"):" PIO",
				(art >> 4) - 1);
			pci_read_config_byte(dev, 0x40, &art);
			p += sprintf(p, "Active:         0x%02x", art >> 4);
			pci_read_config_byte(dev, 0x41, &art);
			p += sprintf(p, "             0x%02x", art >> 4);
			pci_read_config_byte(dev, 0x42, &art);
			p += sprintf(p, "            0x%02x", art >> 4);
			pci_read_config_byte(dev, 0x43, &art);
			p += sprintf(p, "              0x%02x\n", art >> 4);
			pci_read_config_byte(dev, 0x40, &art);
			p += sprintf(p, "Recovery:       0x%02x", art & 0xf);
			pci_read_config_byte(dev, 0x41, &art);
			p += sprintf(p, "             0x%02x", art & 0xf);
			pci_read_config_byte(dev, 0x42, &art);
			p += sprintf(p, "            0x%02x", art & 0xf);
			pci_read_config_byte(dev, 0x43, &art);
			p += sprintf(p, "              0x%02x\n", art & 0xf);
			pci_read_config_byte(dev, 0x49, &uart);
			p += sprintf(p, "reg49h = 0x%02x ", uart);
			pci_read_config_byte(dev, 0x4a, &uart);
			p += sprintf(p, "reg4ah = 0x%02x\n", uart);
			break;
	}
	return p-buffer;/* => must be less than 4k! */
}
