
/*
 * Copyright (c) 2002	Bartlomiej Zolnierkiewicz
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/io.h>

#include "atapci.h"

#define ATAPCI_VERSION	"0.50"

/* taken from lspci */
static char options[] = "hvb:s:d:";

static char help_msg[] = "\
Usage: atapci [<switches>]\n\
-h help\n\
-v version\n\
-b PCI bus speed (between 20000 and 80000)\n\
-s [[<bus>]:][<slot>][.[<func>]]\tshow only devices in selected slots\n\
-d [<vendor>]:[<device>]\t\tshow only selected devices\n";

#ifdef AEC
extern int aec_info(char *, struct pci_dev *);
#endif
#ifdef ALI
extern int ali_info(char *, struct pci_dev *);
#endif
#ifdef AMD
extern int amd_info(char *, struct pci_dev *);
#endif
#ifdef CMD64X
extern int cmd64x_info(char *, struct pci_dev *);
#endif
#ifdef CS5530
extern int cs5530_info(char *, struct pci_dev *);
#endif
#ifdef HPT34X
extern int hpt34x_info(char *, struct pci_dev *);
#endif
#ifdef HPT366
extern int hpt366_info(char *, struct pci_dev *);
#endif
#ifdef INTEL
extern int intel_info(char *, struct pci_dev *);
#endif
#ifdef PDC
extern int pdc202xx_info(char *, struct pci_dev *);
#endif
#ifdef SERVERWORKS
extern int svwks_info(char *, struct pci_dev *);
#endif
#ifdef SIS
extern int sis_info(char *, struct pci_dev *);
#endif
#ifdef VIA
extern int via_info(char *, struct pci_dev *);
#endif


static const struct {
	u16 vendor, device;
	int (*info)(char *, struct pci_dev *);
} atapci_devs[] = {

#ifdef AEC
	{ PCI_VENDOR_ID_ARTOP, PCI_DEVICE_ID_ARTOP_ATP850UF, aec_info },
	{ PCI_VENDOR_ID_ARTOP, PCI_DEVICE_ID_ARTOP_ATP860,   aec_info },
	{ PCI_VENDOR_ID_ARTOP, PCI_DEVICE_ID_ARTOP_ATP860R,  aec_info },
	{ PCI_VENDOR_ID_ARTOP, PCI_DEVICE_ID_ARTOP_ATP865,   aec_info },
	{ PCI_VENDOR_ID_ARTOP, PCI_DEVICE_ID_ARTOP_ATP865R,  aec_info },
#endif
#ifdef ALI
	{ PCI_VENDOR_ID_AL, PCI_DEVICE_ID_AL_M5229, ali_info },
#endif
#ifdef AMD
	{ PCI_VENDOR_ID_AMD,    PCI_DEVICE_ID_AMD_COBRA_7401,    amd_info },
	{ PCI_VENDOR_ID_AMD,    PCI_DEVICE_ID_AMD_VIPER_7409,    amd_info },
	{ PCI_VENDOR_ID_AMD,    PCI_DEVICE_ID_AMD_VIPER_7411,    amd_info },
	{ PCI_VENDOR_ID_AMD,    PCI_DEVICE_ID_AMD_OPUS_7441,     amd_info },
	{ PCI_VENDOR_ID_AMD,    PCI_DEVICE_ID_AMD_8111_IDE,      amd_info },
	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_NFORCE_IDE, amd_info },
#endif
#ifdef CMD64X
	{ PCI_VENDOR_ID_CMD, PCI_DEVICE_ID_CMD_643, cmd64x_info },
	{ PCI_VENDOR_ID_CMD, PCI_DEVICE_ID_CMD_646, cmd64x_info },
	{ PCI_VENDOR_ID_CMD, PCI_DEVICE_ID_CMD_648, cmd64x_info },
	{ PCI_VENDOR_ID_CMD, PCI_DEVICE_ID_CMD_649, cmd64x_info },
#endif
#ifdef CS5530
	{ PCI_VENDOR_ID_CYRIX, PCI_DEVICE_ID_CYRIX_5530_IDE, cs5530_info },
#endif
#ifdef HPT34X
	{ PCI_VENDOR_ID_TTI, PCI_DEVICE_ID_TTI_HPT343, hpt34x_info },
#endif
#ifdef HPT366
	{ PCI_VENDOR_ID_TTI, PCI_DEVICE_ID_TTI_HPT366, hpt366_info },
	{ PCI_VENDOR_ID_TTI, PCI_DEVICE_ID_TTI_HPT372, hpt366_info },
	{ PCI_VENDOR_ID_TTI, PCI_DEVICE_ID_TTI_HPT302, hpt366_info },
	{ PCI_VENDOR_ID_TTI, PCI_DEVICE_ID_TTI_HPT371, hpt366_info },
	{ PCI_VENDOR_ID_TTI, PCI_DEVICE_ID_TTI_HPT374, hpt366_info },
#endif
#ifdef INTEL
	{ PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82801DB_9,  intel_info },
	{ PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82801CA_11, intel_info },
	{ PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82801CA_10, intel_info },
	{ PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82801E_9,   intel_info },
	{ PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82801BA_9,  intel_info },
	{ PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82801BA_8,  intel_info },
	{ PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82801AB_1,  intel_info },
	{ PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82801AA_1,  intel_info },
	{ PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82372FB_1,  intel_info },
	{ PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82443MX_1,  intel_info },
	{ PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82371AB,    intel_info },
	{ PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82371SB_1,  intel_info },
	{ PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82371FB_1,  intel_info },
	{ PCI_VENDOR_ID_EFAR,  PCI_DEVICE_ID_EFAR_SLC90E66_1,  intel_info },
#endif
#ifdef PDC
	{ PCI_VENDOR_ID_PROMISE, PCI_DEVICE_ID_PROMISE_20246, pdc202xx_info },
	{ PCI_VENDOR_ID_PROMISE, PCI_DEVICE_ID_PROMISE_20262, pdc202xx_info },
	{ PCI_VENDOR_ID_PROMISE, PCI_DEVICE_ID_PROMISE_20263, pdc202xx_info },
	{ PCI_VENDOR_ID_PROMISE, PCI_DEVICE_ID_PROMISE_20265, pdc202xx_info },
	{ PCI_VENDOR_ID_PROMISE, PCI_DEVICE_ID_PROMISE_20267, pdc202xx_info },
#endif
#ifdef SERVERWORKS
	{ PCI_VENDOR_ID_SERVERWORKS, PCI_DEVICE_ID_SERVERWORKS_OSB4IDE, svwks_info },
	{ PCI_VENDOR_ID_SERVERWORKS, PCI_DEVICE_ID_SERVERWORKS_CSB5IDE, svwks_info },
	{ PCI_VENDOR_ID_SERVERWORKS, PCI_DEVICE_ID_SERVERWORKS_CSB6IDE, svwks_info },
#endif
#ifdef SIS
	{ PCI_VENDOR_ID_SI, PCI_DEVICE_ID_SI_5513, sis_info },
#endif
#ifdef VIA
	{ PCI_VENDOR_ID_VIA,   PCI_DEVICE_ID_VIA_82C576_1, via_info },
	{ PCI_VENDOR_ID_VIA,   PCI_DEVICE_ID_VIA_82C586_1, via_info },
#endif
};


static struct pci_access *pci_devs;

struct pci_access *pci_setup(void)
{
	struct pci_access *pacc;

	pacc = pci_alloc();
	pci_init(pacc);
	pci_scan_bus(pacc);

	return pacc;
}

struct pci_dev *pci_find_device(unsigned int vendor, unsigned int device,
				struct pci_dev *from)
{
	struct pci_dev *dev = from ? from : (struct pci_dev *)pci_devs->devices;
	for(; dev; dev=dev->next) {
		pci_fill_info(dev, PCI_FILL_IDENT | PCI_FILL_BASES);
		if ((dev->vendor_id == vendor) && (dev->device_id == device))
			return dev;
	}
	return NULL;
}

int generic_info(char *buffer, struct pci_dev *dev)
{
	char *p = buffer;
	u32 bibma = pci_resource_start(dev, 4);
	u8 c0 = 0, c1 = 0;

        /*
         * bibma+0x2 and bibma+0xa are byte
	 * registers to investigate
         */
	if(iopl(3))
		need_root();
	c0 = inb_p((u16)bibma + 0x02);
	c1 = inb_p((u16)bibma + 0x0a);
	iopl(0);

	p += sprintf(p, "--------------- Primary Channel ---------------- Secondary Channel -------------\n");
	p += sprintf(p, "                %sabled                         %sabled\n",
			(c0&0x80) ? "dis" : " en",
			(c1&0x80) ? "dis" : " en");
	p += sprintf(p, "--------------- drive0 --------- drive1 -------- drive0 ---------- drive1 ------\n");
	p += sprintf(p, "DMA enabled:    %s              %s             %s               %s\n",
			(c0&0x20) ? "yes" : "no ",
			(c0&0x40) ? "yes" : "no ",
			(c1&0x20) ? "yes" : "no ",
			(c1&0x40) ? "yes" : "no ");

	return p-buffer;	/* => must be less than 4k! */
}

void need_root(void)
{
	fprintf(stderr, "ATAPCI needs to be run as root.\n");
	exit(EXIT_FAILURE);
}

int pcibus = 33333;

int main(int argc, char **argv)
{
	unsigned int c;
	struct pci_filter filter;
	struct pci_dev *dev;
	char *p, *msg = NULL;
	int i;

	pci_filter_init(NULL, &filter);
	while ((i = getopt(argc, argv, options)) != -1)
		switch (i) {
			case 's':
				msg = pci_filter_parse_slot(&filter, optarg);
				break;
			case 'd':
				msg = pci_filter_parse_id(&filter, optarg);
				break;
			case 'b':
				pcibus = atoi(optarg);
				break;
			case 'h':
				fprintf(stdout, help_msg);
				return 0;
			case 'v':
				printf("atapci version %s\n", ATAPCI_VERSION);
				return 0;
			default:
				break;
		}

	if (msg) {
		printf("%s\n", msg);
		return -1;
	}

	p = (char *)malloc(4096);
	if (!p)
		return -1;

	if (pcibus < 20000 || pcibus > 80000)
		pcibus = 33333;
	printf("pcibus = %u\n", pcibus);

    	pci_devs = pci_setup();
	for(dev=pci_devs->devices; dev; dev=dev->next) {
		if (!pci_filter_match(&filter, dev))
			continue;
		pci_fill_info(dev, PCI_FILL_IDENT | PCI_FILL_BASES);
		for (i = 0; i < (int)ARRAY_SIZE(atapci_devs); i++) {
			if ((dev->vendor_id == atapci_devs[i].vendor) &&
			    (dev->device_id == atapci_devs[i].device)) {
				c = pci_read_word(dev, PCI_CLASS_DEVICE);
				printf( "%02x:%02x.%d vendor=%04x device=%04x "
					"class=%04x irq=%d base4=%lx\n",
					dev->bus, dev->dev, dev->func,
					dev->vendor_id, dev->device_id,
					c, dev->irq, dev->base_addr[4]);
				(void) atapci_devs[i].info(p, dev);
				printf("%s", p);
				break;
			}
		}
	}
	pci_cleanup(pci_devs);
	return 0;
}
