
/*
 * original kernel code by:	Andre Hedrick
 *				Sun Microsystems, Inc.
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/io.h>

#include "atapci.h"

static unsigned int hpt_revision(struct pci_dev *dev)
{
	unsigned int class_rev;
	pci_read_config_dword(dev, PCI_CLASS_REVISION, &class_rev);
	class_rev &= 0xff;

	switch(dev->device_id) {
		case PCI_DEVICE_ID_TTI_HPT374:
			class_rev = PCI_DEVICE_ID_TTI_HPT374; break;
		case PCI_DEVICE_ID_TTI_HPT371:
			class_rev = PCI_DEVICE_ID_TTI_HPT371; break;
		case PCI_DEVICE_ID_TTI_HPT302:
			class_rev = PCI_DEVICE_ID_TTI_HPT302; break;
		case PCI_DEVICE_ID_TTI_HPT372:
			class_rev = PCI_DEVICE_ID_TTI_HPT372; break;
		default:
			break;
	}
	return class_rev;
}

static unsigned int hpt_min_rev(struct pci_dev *dev, unsigned int rev)
{
	return (hpt_revision(dev) >= rev) ? 1 : 0;
}

int hpt366_info(char *buffer, struct pci_dev *dev)
{
	char *p	= buffer;
	char *hpt_devs[] = { "366", "366",  "368",
			     "370", "370A", "372",
			     "302", "371",  "374" };

	unsigned short iobase = pci_resource_start(dev, 4);
	u32 class_rev = hpt_revision(dev);
	u8 c0, c1;

	p += sprintf(p, "Chipset: HPT%s\n", class_rev < ARRAY_SIZE(hpt_devs) ?
			hpt_devs[class_rev] : "unknown");
	p += sprintf(p, "--------------- Primary Channel "
			"--------------- Secondary Channel "
			"--------------\n");

	/* get the bus master status registers */
	if(iopl(3))
		need_root();
	c0 = inb_p(iobase + 0x2);
	c1 = inb_p(iobase + 0xa);
	iopl(0);
	p += sprintf(p, "Enabled:        %s"
				"                             %s\n",
		(c0 & 0x80) ? "no" : "yes",
		(c1 & 0x80) ? "no" : "yes");

	if (hpt_min_rev(dev, 3)) {
		u8 cbl;

		if(iopl(3))
			need_root();
		cbl = inb_p(iobase + 0x7b);
		outb_p(cbl | 1, iobase + 0x7b);
		outb_p(cbl & ~1, iobase + 0x7b);
		cbl = inb_p(iobase + 0x7a);
		iopl(0);

		p += sprintf(p, "Cable:          ATA-%d"
				"                          ATA-%d\n",
			(cbl & 0x02) ? 33 : 66,
			(cbl & 0x01) ? 33 : 66);
		p += sprintf(p, "\n");
	}

	p += sprintf(p, "--------------- drive0 --------- drive1 "
			"------- drive0 ---------- drive1 -------\n");
	p += sprintf(p, "DMA capable:    %s              %s" 
			"            %s               %s\n",
		(c0 & 0x20) ? "yes" : "no ", 
		(c0 & 0x40) ? "yes" : "no ",
		(c1 & 0x20) ? "yes" : "no ", 
		(c1 & 0x40) ? "yes" : "no ");

	{
		u8 c2, c3;
		/* older revs don't have these registers mapped
		 * into io space */
		pci_read_config_byte(dev, 0x43, &c0);
		pci_read_config_byte(dev, 0x47, &c1);
		pci_read_config_byte(dev, 0x4b, &c2);
		pci_read_config_byte(dev, 0x4f, &c3);

		p += sprintf(p, "Mode:           %s             %s"
				"           %s              %s\n",
			(c0 & 0x10) ? "UDMA" : (c0 & 0x20) ? "DMA " :
				(c0 & 0x80) ? "PIO " : "off ",
			(c1 & 0x10) ? "UDMA" : (c1 & 0x20) ? "DMA " :
				(c1 & 0x80) ? "PIO " : "off ",
			(c2 & 0x10) ? "UDMA" : (c2 & 0x20) ? "DMA " :
				(c2 & 0x80) ? "PIO " : "off ",
			(c3 & 0x10) ? "UDMA" : (c3 & 0x20) ? "DMA " :
				(c3 & 0x80) ? "PIO " : "off ");
	}
	p += sprintf(p, "\n");

	return p-buffer;/* => must be less than 4k! */
}
